package org.openmuc.jdlms.internal;

import java.text.MessageFormat;

import org.openmuc.jdlms.internal.asn1.axdr.types.AxdrInteger;

public class DlmsEnumParser {
    public static <T extends Enum<T> & DlmsEnumeration> T enumValueFrom(long code, Class<T> enumClass) {
        return enumValueFrom(new AxdrInteger(code), enumClass);
    }

    public static <T extends Enum<T> & DlmsEnumeration, M extends AxdrInteger> T enumValueFrom(M code,
            Class<T> enumClass) {
        T[] enumConstants = enumClass.getEnumConstants();

        long maskedCode = code.getValue() & 0xFFFFFFFFFFFFFFFFl;
        for (T constant : enumConstants) {
            if (constant.getCode() == maskedCode) {
                return constant;
            }
        }

        // TODO
        String message = MessageFormat.format("No constant with code {0} in {1}.", maskedCode,
                enumClass.getSimpleName());
        throw new IllegalArgumentException(message);

    }

    private DlmsEnumParser() {
    }
}
